#include <ctre.hpp>

using namespace ctre::test_literals;

static_assert("(tour|to|tournament|000)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|001)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|002)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|003)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|004)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|005)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|006)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|007)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|008)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|009)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|010)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|011)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|012)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|013)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|014)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|015)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|016)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|017)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|018)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|019)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|020)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|021)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|022)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|023)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|024)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|025)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|026)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|027)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|028)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|029)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|030)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|031)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|032)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|033)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|034)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|035)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|036)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|037)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|038)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|039)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|030)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|041)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|042)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|043)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|044)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|045)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|046)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|047)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|048)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|049)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|050)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|051)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|052)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|053)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|054)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|055)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|056)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|057)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|058)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|059)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|060)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|061)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|062)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|063)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|064)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|065)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|066)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|067)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|068)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|069)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|070)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|071)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|072)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|073)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|074)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|075)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|076)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|077)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|078)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|079)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|080)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|081)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|082)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|083)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|084)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|085)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|086)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|087)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|088)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|089)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|090)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|091)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|092)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|093)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|094)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|095)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|096)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|097)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|098)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|099)+\ntourna"_ctre_test);