#include <ctre.hpp>

using namespace ctre::test_literals;

static_assert("(tour|to|tournament|000)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|001)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|002)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|003)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|004)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|005)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|006)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|007)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|008)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|009)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|010)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|011)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|012)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|013)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|014)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|015)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|016)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|017)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|018)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|019)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|020)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|021)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|022)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|023)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|024)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|025)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|026)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|027)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|028)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|029)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|030)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|031)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|032)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|033)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|034)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|035)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|036)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|037)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|038)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|039)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|030)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|041)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|042)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|043)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|044)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|045)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|046)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|047)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|048)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|049)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|050)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|051)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|052)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|053)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|054)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|055)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|056)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|057)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|058)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|059)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|060)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|061)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|062)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|063)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|064)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|065)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|066)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|067)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|068)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|069)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|070)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|071)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|072)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|073)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|074)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|075)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|076)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|077)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|078)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|079)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|080)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|081)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|082)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|083)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|084)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|085)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|086)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|087)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|088)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|089)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|090)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|091)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|092)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|093)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|094)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|095)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|096)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|097)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|098)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|099)+\ntourna"_ctre_test);

static_assert("(tour|to|tournament|100)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|101)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|102)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|103)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|104)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|105)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|106)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|107)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|108)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|109)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|110)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|111)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|112)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|113)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|114)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|115)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|116)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|117)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|118)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|119)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|120)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|121)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|122)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|123)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|124)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|125)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|126)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|127)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|128)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|129)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|130)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|131)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|132)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|133)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|134)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|135)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|136)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|137)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|138)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|139)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|130)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|141)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|142)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|143)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|144)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|145)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|146)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|147)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|148)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|149)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|150)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|151)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|152)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|153)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|154)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|155)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|156)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|157)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|158)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|159)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|160)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|161)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|162)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|163)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|164)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|165)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|166)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|167)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|168)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|169)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|170)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|171)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|172)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|173)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|174)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|175)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|176)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|177)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|178)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|179)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|180)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|181)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|182)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|183)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|184)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|185)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|186)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|187)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|188)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|189)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|190)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|191)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|192)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|193)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|194)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|195)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|196)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|197)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|198)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|199)+\ntourna"_ctre_test);

static_assert("(tour|to|tournament|200)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|201)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|202)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|203)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|204)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|205)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|206)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|207)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|208)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|209)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|210)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|211)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|212)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|213)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|214)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|215)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|216)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|217)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|218)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|219)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|220)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|221)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|222)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|223)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|224)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|225)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|226)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|227)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|228)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|229)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|230)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|231)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|232)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|233)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|234)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|235)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|236)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|237)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|238)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|239)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|230)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|241)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|242)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|243)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|244)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|245)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|246)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|247)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|248)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|249)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|250)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|251)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|252)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|253)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|254)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|255)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|256)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|257)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|258)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|259)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|260)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|261)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|262)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|263)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|264)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|265)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|266)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|267)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|268)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|269)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|270)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|271)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|272)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|273)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|274)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|275)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|276)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|277)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|278)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|279)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|280)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|281)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|282)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|283)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|284)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|285)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|286)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|287)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|288)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|289)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|290)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|291)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|292)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|293)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|294)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|295)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|296)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|297)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|298)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|299)+\ntourna"_ctre_test);

static_assert("(tour|to|tournament|300)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|301)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|302)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|303)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|304)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|305)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|306)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|307)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|308)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|309)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|310)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|311)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|312)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|313)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|314)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|315)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|316)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|317)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|318)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|319)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|320)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|321)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|322)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|323)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|324)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|325)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|326)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|327)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|328)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|329)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|330)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|331)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|332)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|333)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|334)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|335)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|336)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|337)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|338)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|339)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|330)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|341)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|342)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|343)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|344)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|345)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|346)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|347)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|348)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|349)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|350)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|351)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|352)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|353)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|354)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|355)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|356)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|357)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|358)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|359)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|360)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|361)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|362)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|363)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|364)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|365)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|366)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|367)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|368)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|369)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|370)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|371)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|372)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|373)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|374)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|375)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|376)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|377)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|378)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|379)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|380)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|381)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|382)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|383)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|384)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|385)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|386)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|387)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|388)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|389)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|390)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|391)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|392)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|393)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|394)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|395)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|396)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|397)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|398)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|399)+\ntourna"_ctre_test);

static_assert("(tour|to|tournament|400)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|401)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|402)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|403)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|404)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|405)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|406)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|407)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|408)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|409)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|410)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|411)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|412)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|413)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|414)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|415)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|416)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|417)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|418)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|419)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|420)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|421)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|422)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|423)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|424)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|425)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|426)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|427)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|428)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|429)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|430)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|431)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|432)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|433)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|434)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|435)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|436)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|437)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|438)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|439)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|430)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|441)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|442)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|443)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|444)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|445)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|446)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|447)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|448)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|449)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|450)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|451)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|452)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|453)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|454)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|455)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|456)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|457)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|458)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|459)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|460)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|461)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|462)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|463)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|464)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|465)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|466)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|467)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|468)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|469)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|470)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|471)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|472)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|473)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|474)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|475)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|476)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|477)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|478)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|479)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|480)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|481)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|482)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|483)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|484)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|485)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|486)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|487)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|488)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|489)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|490)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|491)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|492)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|493)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|494)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|495)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|496)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|497)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|498)+\ntourna"_ctre_test);
static_assert("(tour|to|tournament|499)+\ntourna"_ctre_test);