#include <ctre.hpp>
using namespace ctre::test_literals;
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);
static_assert("(tour|to|tournament)+\ntourna"_pcre_test);